rem
rem $Header: blocking.sql,v 6002607.2 89/10/12 19:57:20 cyang Exp $ blocking.sql Copyr (c) 1989 Oracle
rem
/* This script creates a view called blocking_locks that can be queried
 * to determine which locks are blocking the system.  It is mostly useful
 * for finding processes that have died holding locks, and that for some
 * reason have not been cleaned up by PMON.
 *  
 * It finds all the locks and latches that have processes waiting on them
 * that are held by a process that is not waiting on another lock.  In
 * other words it finds the locks that are at the top of the wait-for
 * graph.
 *  
 * The blocking_locks view also gives information on the process that is
 * holding the lock that is blocking the system.
 *  
 * Note that this view will not work if there is some sort of circular
 * dependence between processes holding locks.  That is, it will not work
 * in the case that there is a deadlock that is not detected automatically
 * by the system.  Also, the results of the view are not dependable if there
 * are a lot of lock transitions.  It is really only useful for the case when
 * the system is wedged.
 *  
 * You must have access to the fixed tables (owned by sys) in order to
 * use the blocking_locks view.
 */

drop view ALL_LOCKS_VIEW;

/* All_locks_view has a row for each lock or latch that is being held, and 
 * one row for each outstanding request for a lock or latch.
 * The columns  of all_locks_view are:
 *   type    - type of lock (DDL, LATCH, etc.)
 *   lmode   - mode the lock is currently held in by process pid
 *   pid     - oracle pid of process that this row describes
 *   id1     - lock specific identifier of the lock
 *   id2     - lock specific identifier of the lock
 *   request - mode that the lock is being requested in by process pid
 */
create view ALL_LOCKS_VIEW as
  select type, lmode, pid, to_char(id1) id1, to_char(id2) id2, request 
      from v$lock                /* processes waiting on or holding enqueues */
 union
  select 'LATCH', 6, pid, rawtohex(laddr), ' ', 0   /* procs holding latches */
      from v$latchholder                      /* 6 = exclusive, 0 = not held */
 union
  select 'LATCH', 0, pid, latchwait,' ',6          /* procs waiting on latch */
      from v$process
    where latchwait is not null;

/* drop old obsolete view */
drop view BLOCKING_LOCKS1;

drop view BLOCKING_LOCKS;

/* Locks being requested by some process, that are held by processes 
 * that are not waiting to acquire some other lock.  In other words 
 * the locks  at the top of the WAIT-FOR tree.  Also some information
 * about the holder of the lock.
 */

create view BLOCKING_LOCKS as
select h.type		lock_type,
	decode( h.lmode, 
		0, 'NONE',           /* Mon Lock equivalent */
                1, 'Null Mode',      /* N */
		2, 'Row-S (SS)',     /* L */
		3, 'Row-X (SX)',     /* R */
		4, 'Share (S)',      /* S */
		5, 'S/Row-X (SSX)',  /* C */
		6, 'Exclusive (X)',  /* X */
		h.lmode)
			lock_mode,
	h.pid		orcl_pid,                       /* oracle process id */
	s.username	orcl_user,                       /* oracle user name */
	p.spid 		os_pid,                    /* OS specific process id */
	p.username 	os_user,                         /* OS specific name */
	program,                             /* program that user is running */
	terminal,                             /* terminal that user is using */
	h.id1 rsrc_id1,    /* the value of these depends on the type of lock */
	h.id2 rsrc_id2
  from v$session s, v$process p, all_locks_view h, all_locks_view r
 where  r.request  >  1     /* someone is waiting for (requesting) the lock  */
  and   h.lmode    >  1                             /* h is holding the lock */
  and   h.type     =  r.type
  and   h.id1      =  r.id1
  and   h.id2      =  r.id2
  and   h.pid      =  p.pid
  and   p.lockwait is null
  and   p.latchwait is null
  and   p.addr     =   s.paddr;


