rem
rem $Header: locktree.sql,v 6002700.1 89/10/12 19:55:18 cyang Exp $ locktree.sql Copyr (c) 1989 Oracle
rem

rem Print out the lock wait-for graph in a tree structured fashion.
rem   
rem  This script prints  the  processes in   the system  that  are waiting for
rem  locks,  and the locks that they  are waiting for.   The  printout is tree
rem  structured.  If a processid is printed immediately below and to the right
rem  of another process, then it is waiting for that process.  The process ids
rem  printed at the left hand side of the page are  the ones  that everyone is
rem  waiting for.
rem   
rem  For example, in the following printout process 9 is waiting for
rem  process 8, 7 is waiting for 9, and 10 is waiting for 9.
rem   
rem  PROCESS WAITING   TYPE MODE REQUESTED    MODE HELD         LOCK ID1 LOCK ID2
rem  ----------------- ---- ----------------- ----------------- -------- --------
rem  8                 NONE None              None              0         0
rem     9              TX   Share (S)         Exclusive (X)     65547     16
rem        7           RW   Exclusive (X)     S/Row-X (SSX)     33554440  2
rem        10          RW   Exclusive (X)     S/Row-X (SSX)     33554440  2
rem   
rem  The lock information to the right of the process id describes the lock
rem  that the process is waiting for (not the lock it is holding).
rem   
rem  You must have the blocking.sql script loaded for this  script to work (it
rem  is in world$:blocking.sql).
rem   
rem  Note that  this is a  script and not a  set  of view  definitions because
rem  connect-by is used in the implementation and therefore  a temporary table
rem  is created and dropped since you cannot do a join in a connect-by.
rem   
rem  This script has two  small disadvantages.  One, a  table is created  when
rem  this  script is run.   To create  a table   a  number of   locks must  be
rem  acquired. This  might cause the process running  the script to get caught
rem  in the lock problem it is trying to diagnose.  Two, if a process waits on
rem  a lock held by more than one process (share lock) then the wait-for graph
rem  is no longer a tree  and the  conenct-by will show the process  (and  any
rem  processes waiting on it) several times.
rem 

rem Select all pids waiting for a lock, the lock they are waiting on, and the
rem  pid of the process that holds the lock.
rem   UNION
rem  The pids of all processes holding locks that someone is waiting on that
rem  are not themselves waiting for locks. These are included so that the roots
rem  of the wait for graph (the processes holding things up) will be displayed.
rem   
rem     pid    - pid of requesting process
rem     type   - type of lock being requested
rem     id1    - id1 of lock being requested (value is lock type specific)
rem     id2    - id2 of lock being requested (value is lock type specific)
rem     req    - mode lock is being requested in
rem     hpid   - pid of process holding the lock
rem     hmod   - mode the lock is held in
rem 
rem  NOTE:  For interpratation of id1 and id2 please refer to notes$:ksq.doc
rem
create table LOCK_HOLDERS 
(
  pid number(18),
  type char(4),
  id1 char(10),
  id2 char(10),
  req number,
  hpid number,
  hmod number
);

insert into lock_holders 
select r.pid, r.type, r.id1, r.id2, r.request req, h.pid hpid, h.lmode hmod
  from all_locks_view h, all_locks_view r
 where  r.request  >  1            /* R is waiting for (requesting) the lock */
  and   h.lmode    >  1                             /* H is holding the lock */
  and   h.type     =  r.type        /* H is holding the lock R is requesting */
  and   h.id1      =  r.id1
  and   h.id2      =  r.id2
union
select unique
       h.pid, 'NONE', '0', '0', 0, -1, 0
  from v$process p, all_locks_view h, all_locks_view r
 where  r.request  >  1            /* R is waiting for (requesting) the lock */
  and   h.lmode    >  1                             /* h is holding the lock */
  and   h.type     =  r.type        /* H is holding the lock R is requesting */
  and   h.id1      =  r.id1
  and   h.id2      =  r.id2
  and   h.pid      =  p.pid                                /* H is process P */
  and   p.lockwait is null                    /* P is not waiting for a lock */
  and   p.latchwait is null;

set charwidth 17;

rem Print out the result in a tree structured fashion

select lpad(' ',3*(level-1)) || pid "PROCESS WAITING",
       type type, 
       decode( req, 
		0, 'None',           /* Mon Lock equivalent */
                1, 'Null Mode',      /* N */
		2, 'Row-S (SS)',     /* L */
		3, 'Row-X (SX)',     /* R */
		4, 'Share (S)',      /* S */
		5, 'S/Row-X (SSX)',  /* C */
		6, 'Exclusive (X)',  /* X */
		req) "MODE REQUESTED",
       decode( hmod, 
		0, 'None',           /* Mon Lock equivalent */
                1, 'Null Mode',      /* N */
		2, 'Row-S (SS)',     /* L */
		3, 'Row-X (SX)',     /* R */
		4, 'Share (S)',      /* S */
		5, 'S/Row-X (SSX)',  /* C */
		6, 'Exclusive (X)',  /* X */
		hmod) "MODE HELD",
       id1 "LOCK ID1", id2 "LOCK ID2"
 from lock_holders
connect by  prior pid = hpid
  start with hpid = -1;

drop table lock_holders;
